
#include "StaticConcept.h"
#include "ConceptMap.h"

StaticConcept::StaticConcept(Concept& concept)
  : mConcept(concept)
{
  mConcept.addStaticConcept(this);
}

ErrorCode
StaticConcept::connect(StaticConcept& connector,const String& port)
{
  Concept* portConcept = findGlobalConcept(port);
  if (!portConcept) return eConceptDoesNotExists;

  Connection* con = new Connection(connector,*portConcept);
  for (list<Connection*>::iterator i = mConnections.begin();
       i != mConnections.end();
       ++i)
    if (sameConnection(con,*i)) return eConnectionExists;

  ErrorCode result = connector.connectPort(*portConcept);
  if (result != eSuccess) return result;

  mConnections.push_back(con);
  return eSuccess;
}

bool
StaticConcept::sameConnection(Connection* con1,Connection* con2)
{
  cout << "sameConnection" << endl;
  return ((con1->connector.getName() == con2->connector.getName()) &&
	  (con1->port.getName() == con2->port.getName()));
}

ErrorCode
StaticConcept::definePort(StaticConcept& sc)
{
  mConcept.addPort(&sc.mConcept);

  return sc.connect(*this,sc.getName());
}

ErrorCode
StaticConcept::connectPort(Concept& portConcept)
{
  if (!mConcept.portExists(&portConcept)) return ePortDoesNotExists;

  for (list<Concept*>::iterator i = mOccupiedPorts.begin();
       i != mOccupiedPorts.end();
       ++i)
    if ((*i)->getName() == portConcept.getName()) return ePortIsOccupied;

  mOccupiedPorts.push_back(&portConcept);
  return eSuccess;
}
